# -*- coding:Utf-8 -*-

# ==================================================
#  >> IMPORTATIONS                                 #
# ==================================================


import es
import langlib
import usermsg
import playerlib
import gamethread

import os
import re
import random
import configobj

import psyco
psyco.full()


# ==================================================
#  >> INFORMATIONS                                 #
# ==================================================


temp = {
   'name'       : 'Gore Mod',
   'version'    : '2.0',
   'date'       : '2010-04-11',
   'author'     : 'L\'In20Cible',
   'description': 'Simule des effets Gore sur votre serveur.',
   'url'        : 'http://addons.eventscripts.com/addons/view/goremod',
   'basename'   : 'goremod'
   }
   
informations = es.AddonInfo()
for information in temp:
   informations[information] = temp[information]
   
es.ServerVar('gm_version', (informations['version'] + ' by ' + informations['author'])).makepublic()


# ==================================================
#  >> INITIALISATION                               #
# ==================================================


def load():
   Initialization()
   
   
def Initialization():
   global models, players, options, translations
   models = dict()
   try:
      models = ConfigurationFileParser(GetConfigurationFilePath('models.ini'))
   except:
      pass
   players = list()
   options = {
      'enable'               : ('1', 'Active/désactive le GoreMod.'),
      'blood'                : ('1', 'Active/désactive le surplus de sang.'),
      'blood_alien'          : ('0', 'Active/désactive le sang alien.'),
      'blood_amount'         : ('10', 'Définit la quantité de surplus de sang'),
      'decapitation'         : ('1', 'Active/désactive la décapitation.'),
      'decapitation_bleeding': ('1', 'Active/désactive les saignements des joueurs décapités.'),
      'decapitation_weapons' : ('world,worldspawn,point_hurt,hegrenade,flashbang', 'Définit les armes avec lesquels la victime sera décapité.'),
      'decapitation_sound'   : ('goremod/decapitation.wav', 'Définit le son joué lors d\'une décapitation.'),
      'explosion'            : ('1', 'Active/désactive l\'explosion.'),
      'explosion_weapons'    : ('world,worldspawn,point_hurt,knife,hegrenade,awp', 'Définit les armes qui exploseront leurs victimes.'),
      'explosion_sound'      : ('goremod/explosion.wav', 'Définit le joué lors d\'une explosion.')
      }
   for option in options:
      options[option] = es.ServerVar(('gm_' + option), options[option][0], options[option][1])
   filepath = GetConfigurationFilePath('configuration.cfg')
   if os.path.isfile(filepath):
      es.mexec(filepath.replace(str(es.ServerVar('eventscripts_gamedir')).replace(os.sep, '/'), '..'))
   filepath = GetConfigurationFilePath('downloads.txt')
   if os.path.isfile(filepath) and str(es.ServerVar('eventscripts_currentmap')):
      regex = re.compile('//.*|^(( |\t)+)|(( |\t)+)$|(\r)+|(\n)+')
      for fileline in open(filepath):
         fileline = regex.sub('', fileline).strip()
         if not fileline:
            continue
         es.stringtable('downloadables', fileline)
   translations = TranslationsAreMissingCallBack
   try:
      translations = langlib.Strings(GetConfigurationFilePath('translations.ini'))
   except:
      pass
      
      
# ==================================================
#  >> CLASSES                                      #
# ==================================================


class ConfigurationFileParser(configobj.ConfigObj):
   def __getitem__(self, itemname):
      if itemname not in self:
         return ''
      return super(ConfigurationFileParser, self).__getitem__(itemname)
      
      
# ==================================================
#  >> UTILITAIRES                                  #
# ==================================================


def GetConfigurationFilePath(filename):
   return os.path.join(es.getAddonPath(informations.basename), 'configuration', filename).replace(os.sep, '/')
   
   
def GetPlayerHitGroupLocation(userid, hitgroup):
   userid = int(userid)
   hitgroup = int(hitgroup)
   if not es.exists('userid', userid) or (hitgroup not in xrange(-1,  8)):
      return
   x, y, z = es.getplayerlocation(userid)
   if hitgroup == 1:
      z = (z + 65)
   elif hitgroup in (2, 3, 4, 5):
      z = (z + 45)
   elif hitgroup in (6, 7):
      z = (z + 25)
   if (es.getplayerprop(userid, 'CBasePlayer.localdata.m_Local.m_bDucked') % 2):
      z = (z - 20)
   return tuple([x, y, z])
   
   
def CreateBloodEffect(userid, origin, amount):
   userid = int(userid)
   if not es.exists('userid', userid):
      return
   strings = [
      'es_xentcreate %i env_blood' % userid,
      'es_xfire %i env_blood addoutput "amount %i"' % (userid, int(amount)),
      'es_xfire %i env_blood addoutput "color %i"' % (userid, int(options['blood_alien'])),
      'es_xfire %i env_blood addoutput "spawnflags 1321"' % userid,
      'es_xfire %i env_blood addoutput "origin %s %s %s"' % (userid, origin[0], origin[1], origin[2]),
      'es_xfire %i env_blood emitblood' % userid,
      'es_xfire %i env_blood kill' % userid
      ]
   for string in strings:
      es.server.queuecmd(string)
      
      
def CreateShootEffect(userid, entityname, origin, model, amount):
   userid = int(userid)
   if not es.exists('userid', userid):
      return
   es.precachemodel(model)
   string = [
      'es_xentcreate %i env_shooter' % userid,
      'es est_setentname server_var(eventscripts_lastgive) %s' % entityname,
      'es_xfire %i %s addoutput "origin %s %s %s"' % (userid, entityname, origin[0], origin[1], origin[2]),
      'es_xfire %i %s addoutput "m_iGibs %i"' % (userid, entityname, int(amount)),
      'es_xfire %i %s addoutput "m_flVelocity 500"' % (userid, entityname),
      'es_xfire %i %s addoutput "m_flVariance 500"' % (userid, entityname),
      'es_xfire %i %s addoutput "m_flGibLife 120"' % (userid, entityname),
      'es_xfire %i %s addoutput "renderamt 255"' % (userid, entityname),
      'es_xfire %i %s addoutput "rendercolor 255 255 255"' % (userid, entityname),
      'es_xfire %i %s addoutput "disablereceiveshadows 1"' % (userid, entityname),
      'es_xfire %i %s addoutput "shootmodel %s"' % (userid, entityname, model),
      'es_xfire %i %s addoutput "shootsounds -1"' % (userid, entityname),
      'es_xfire %i %s addoutput "simulation 1"' % (userid, entityname),
      'es_xfire %i %s addoutput "gibanglevelocity -250"' % (userid, entityname),
      'es_xfire %i %s addoutput "gibgravityscale 0.1"' % (userid, entityname),
      'es_xfire %i %s addoutput "spawnflags 4"' % (userid, entityname),
      'es_xfire %i %s shoot' % (userid, entityname)
      ]
   for string in string:
      es.server.queuecmd(string)
      
      
def GetPlayerLanguage(userid):
   return langlib.getLangAbbreviation(es.getclientvar(userid, 'cl_language'))
   
   
def PlayerBleed(userid):
   userid = int(userid)
   if not int(options['enable']) or not int(options['decapitation_bleeding']) or not es.exists('userid', userid) or not (userid in players):
      return
   if not es.getplayerprop(userid, 'CBasePlayer.pl.deadflag'):
      if not es.isbot(userid):
         ShakePlayerScreen(userid, 10, 0.5)
      CreateBloodEffect(userid, GetPlayerHitGroupLocation(userid, 1), random.randint(10, 25))
      if random.randint(1, 3) == 2:
         es.trick('entity', 'point_hurt')
         strings = [
            'es_xfire %i point_hurt kill' % userid,
            'es_xfire %i !self addoutput "targetname BleedingPlayers"' % userid,
            'es_xfire %i point_hurt addoutput "damage 1"' %userid,
            'es_xfire %i point_hurt addoutput "damagetarget BleedingPlayers"' % userid,
            'es_xfire %i point_hurt addoutput "damagetype 0"' % userid,
            'es_xfire %i point_hurt hurt' % userid,
            'es_xfire %i player addoutput "targetname NormalPlayers"' % userid
            ]
         for string in strings:
            es.server.queuecmd(string)
      gamethread.delayed(random.random(), PlayerBleed, userid)
   
   
def ShakePlayerScreen(userid, magnitude, time):
   if not es.exists('userid', userid):
      return
   usermsg.shake(userid, magnitude, time)
   
   
# ==================================================
#  >> FONCTIONS D'APPEL                            #
# ==================================================


def TranslationsAreMissingCallBack(string, tokens, language):
   return 'Sorry, unable to read the #lightgreentranslations.ini #defaultfile.'
   
   
# ==================================================
#  >> ÉVÉNEMENTS                                   #
# ==================================================


def es_map_start(event_var):
   Initialization()
   
   
def player_spawn(event_var):
   userid = int(event_var['userid'])
   if userid not in players:
      return
   players.remove(userid)
   
   
def player_death(event_var):
   userid = int(event_var['userid'])
   if not int(options['enable']) and not int(options['explosion']):
      return
   if (event_var['weapon'] in str(options['explosion_weapons']).split(',')) or int(event_var['headshot']) or ((event_var['es_userteam']) == event_var['es_attackerteam']):
      userhandle = es.getplayerhandle(userid)
      for ragdollindex in es.createentitylist('cs_ragdoll'):
         if not es.getindexprop(ragdollindex, 'CCSRagdoll.m_hPlayer') == userhandle:
            continue
         es.server.queuecmd('es_xremove %i' % ragdollindex)
      CreateBloodEffect(userid, GetPlayerHitGroupLocation(userid, 1), random.randint(10, 25))
      es.emitsound('player', userid, options['explosion_sound'], 1.0, 0.5)
      hitgrouplocation = GetPlayerHitGroupLocation(userid, 1)
      CreateShootEffect(userid, ('gm_shooter_explosion_hgibs_' + str(userid)), hitgrouplocation, 'models/gibs/hgibs.mdl', 1)
      CreateShootEffect(userid, ('gm_shooter_explosion_scapula_' + str(userid)), hitgrouplocation, 'models/gibs/hgibs_scapula.mdl', 2)
      CreateShootEffect(userid, ('gm_shooter_explosion_hgibs_spine' + str(userid)), hitgrouplocation, 'models/gibs/hgibs_spine.mdl', 1)
      CreateShootEffect(userid, ('gm_shooter_explosion_hgibs_rib' + str(userid)), hitgrouplocation, 'models/gibs/hgibs_rib.mdl', 4)
      userindex = es.createplayerlist(userid)[userid]['index']
      tokens = {
         'victim'  : es.getplayername(userid),
         'attacker': es.getplayername(event_var['attacker'])
         }
      for playerid in es.getUseridList():
         if es.isbot(playerid):
            continue
         usermsg.saytext2(playerid, userindex, ('[GoreMod] ' + translations('explosion', tokens, GetPlayerLanguage(playerid))))
   if userid not in players:
      return
   players.remove(userid)
   
   
def player_hurt(event_var):
   if not int(options['enable']):
      return
   userid = int(event_var['userid'])
   if int(options['blood']):
      CreateBloodEffect(userid, GetPlayerHitGroupLocation(userid, event_var['hitgroup']), int(options['blood_amount']))
   if not int(options['decapitation']) or (userid in players):
      return
   if ((int(event_var['hitgroup']) == 1) or (event_var['weapon'] in str(options['decapitation_weapons']).split(','))) and not es.getplayerprop(userid, 'CBasePlayer.pl.deadflag'):
      usermodel = es.createplayerlist(userid)[userid]['model']
      if models[usermodel] and (models[usermodel]['body'] and models[usermodel]['head']):
         players.append(userid)
         playerlib.getPlayer(userid).setModel(models[usermodel]['body'])
         ShakePlayerScreen(userid, 100, 1)
         CreateShootEffect(userid, ('gm_shooter_decapitation_' + str(userid)), GetPlayerHitGroupLocation(userid, 1), models[usermodel]['head'], 1)
         userindex = es.createplayerlist(userid)[userid]['index']
         tokens = {
            'victim'  : es.getplayername(userid),
            'attacker': es.getplayername(event_var['attacker'])
            }
         for playerid in es.getUseridList():
            if es.isbot(playerid):
               continue
            usermsg.saytext2(playerid, userindex, ('[GoreMod] ' + translations('decapitation', tokens, GetPlayerLanguage(playerid))))
         if int(options['decapitation_bleeding']):
            gamethread.delayed(random.random(), PlayerBleed, userid)
         es.emitsound('player', userid, options['decapitation_sound'], 1.0, 0.5)
         
         
def hostage_hurt(event_var):
   if not int(options['enable']) and not int(options['blood']):
      return
   x, y, z = es.getindexprop(event_var['hostage'], 'CBaseEntity.m_vecOrigin').split(',')
   z = (float(z) + 50)
   CreateBloodEffect(event_var['userid'], es.createvectorstring(x, y, z).split(','), int(options['blood_amount']))
   
   
def player_say(event_var):
   player_hurt({'userid': event_var['userid'], 'hitgroup': '1', 'dmg_health': '1', 'es_userhealth': '100', 'attacker': '13'})